@@ -8,7 +8,15 @@
 #include <QtCore/QDateTime>
 
 namespace Quotient {
+
+constexpr inline auto EventIdKey = "event_id"_L1;
+constexpr inline auto RoomIdKey = "room_id"_L1;
+constexpr inline auto StateKeyKey = "state_key"_L1;
+constexpr inline auto RedactedCauseKey = "redacted_because"_L1;
+constexpr inline auto RelatesToKey = "m.relates_to"_L1;
+
 class RedactionEvent;
+class EncryptedEvent;
 
 // That check could look into Event and find most stuff already deleted...
 // NOLINTNEXTLINE(cppcoreguidelines-special-member-functions)
@@ -18,7 +26,15 @@ public:
 
     ~RoomEvent() override; // Don't inline this - see the private section
 
+    //! \brief A convenience function to get a display string for an event ID.
+    //! \return id() if the event id is not empty, otherwise transactionId();
+    //!              this is useful to deal with pending and normal events uniformly.
+    //! \sa id(), transactionId()
+    QString displayId() const;
+
+    //! The event_id JSON value for the event.
     QString id() const;
+
     QDateTime originTimestamp() const;
     QString roomId() const;
     QString senderId() const;
@@ -28,7 +44,16 @@ public:
         return _redactedBecause;
     }
     QString redactionReason() const;
+
+    //! The transaction_id JSON value for the event.
     QString transactionId() const;
+
+    // State events are special in Matrix; so isStateEvent() and stateKey() are here,
+    // as an exception. For other event types (including base types), Event::is<>() and
+    // Quotient::is<>() should be used
+
+    bool isStateEvent() const;
+
     QString stateKey() const;
 
     //! \brief Fill the pending event object with the room id
@@ -50,11 +75,9 @@ public:
     //! callback for that in RoomEvent.
     void addId(const QString& newId);
 
-#ifdef Quotient_E2EE_ENABLED
-    void setOriginalEvent(event_ptr_tt<RoomEvent>&& originalEvent);
-    const RoomEvent* originalEvent() const { return _originalEvent.get(); }
+    void setOriginalEvent(event_ptr_tt<EncryptedEvent>&& originalEvent);
+    const EncryptedEvent* originalEvent() const { return _originalEvent.get(); }
     const QJsonObject encryptedJson() const;
-#endif
 
 protected:
     explicit RoomEvent(const QJsonObject& json);
@@ -65,13 +88,14 @@ private:
     // constructors using it and also destructors (with 'using', in particular).
     event_ptr_tt<RedactionEvent> _redactedBecause;
 
-#ifdef Quotient_E2EE_ENABLED
-    event_ptr_tt<RoomEvent> _originalEvent;
-#endif
+    event_ptr_tt<EncryptedEvent> _originalEvent;
 };
 using RoomEventPtr = event_ptr_tt<RoomEvent>;
 using RoomEvents = EventsArray<RoomEvent>;
-using RoomEventsRange = Range<RoomEvents>;
+using RoomEventsRange = std::ranges::subrange<RoomEvents::iterator>;
+
+//! \brief Determine whether a given event type is that of a state event
+QUOTIENT_API bool isStateEvent(const QString& eventTypeId);
 
 } // namespace Quotient
 Q_DECLARE_METATYPE(Quotient::RoomEvent*)
