@@ -10,6 +10,7 @@
 #pragma once
 
 #include "connection.h"
+#include "roommember.h"
 #include "roomstateview.h"
 #include "eventitem.h"
 #include "quotient_common.h"
@@ -18,17 +19,17 @@
 
 #include "events/accountdataevents.h"
 #include "events/encryptedevent.h"
+#include "events/eventrelation.h"
+#include "events/roomcreateevent.h"
 #include "events/roomkeyevent.h"
 #include "events/roommessageevent.h"
-#include "events/roomcreateevent.h"
+#include "events/roompowerlevelsevent.h"
 #include "events/roomtombstoneevent.h"
-#include "events/eventrelation.h"
 
 #include <QtCore/QJsonObject>
 #include <QtGui/QImage>
 
 #include <deque>
-#include <memory>
 #include <utility>
 
 namespace Quotient {
@@ -37,7 +38,8 @@ class Avatar;
 class SyncRoomData;
 class RoomMemberEvent;
 class User;
-class MemberSorter;
+class RoomMember;
+struct MemberSorter;
 class LeaveRoomJob;
 class SetRoomStateWithKeyJob;
 class RedactEventJob;
@@ -115,7 +117,7 @@ private:
 class QUOTIENT_API Room : public QObject {
     Q_OBJECT
     Q_PROPERTY(Connection* connection READ connection CONSTANT)
-    Q_PROPERTY(User* localUser READ localUser CONSTANT)
+    Q_PROPERTY(RoomMember localMember READ localMember CONSTANT)
     Q_PROPERTY(QString id READ id CONSTANT)
     Q_PROPERTY(QString version READ version NOTIFY baseStateLoaded)
     Q_PROPERTY(bool isUnstable READ isUnstable NOTIFY stabilityUpdated)
@@ -136,10 +138,12 @@ class QUOTIENT_API Room : public QObject {
     Q_PROPERTY(bool usesEncryption READ usesEncryption NOTIFY encryption)
 
     Q_PROPERTY(int timelineSize READ timelineSize NOTIFY addedMessages)
-    Q_PROPERTY(QStringList memberNames READ safeMemberNames NOTIFY memberListChanged)
     Q_PROPERTY(int joinedCount READ joinedCount NOTIFY memberListChanged)
     Q_PROPERTY(int invitedCount READ invitedCount NOTIFY memberListChanged)
     Q_PROPERTY(int totalMemberCount READ totalMemberCount NOTIFY memberListChanged)
+    Q_PROPERTY(QList<RoomMember> membersTyping READ membersTyping NOTIFY typingChanged)
+    Q_PROPERTY(QList<RoomMember> otherMembersTyping READ otherMembersTyping NOTIFY typingChanged)
+    Q_PROPERTY(int localMemberEffectivePowerLevel READ memberEffectivePowerLevel NOTIFY changed)
 
     Q_PROPERTY(bool displayed READ displayed WRITE setDisplayed NOTIFY
                    displayedChanged)
@@ -147,31 +151,22 @@ class QUOTIENT_API Room : public QObject {
                    setFirstDisplayedEventId NOTIFY firstDisplayedEventChanged)
     Q_PROPERTY(QString lastDisplayedEventId READ lastDisplayedEventId WRITE
                    setLastDisplayedEventId NOTIFY lastDisplayedEventChanged)
-    //! \deprecated since 0.7
-    Q_PROPERTY(QString readMarkerEventId READ readMarkerEventId WRITE
-                   markMessagesAsRead NOTIFY readMarkerMoved)
     Q_PROPERTY(QString lastFullyReadEventId READ lastFullyReadEventId WRITE
                    markMessagesAsRead NOTIFY fullyReadMarkerMoved)
-    //! \deprecated since 0.7
-    Q_PROPERTY(bool hasUnreadMessages READ hasUnreadMessages NOTIFY
-                   partiallyReadStatsChanged STORED false)
-    //! \deprecated since 0.7
-    Q_PROPERTY(int unreadCount READ unreadCount NOTIFY partiallyReadStatsChanged
-                   STORED false)
     Q_PROPERTY(qsizetype highlightCount READ highlightCount
                    NOTIFY highlightCountChanged)
     Q_PROPERTY(qsizetype notificationCount READ notificationCount
                    NOTIFY notificationCountChanged)
     Q_PROPERTY(EventStats partiallyReadStats READ partiallyReadStats NOTIFY partiallyReadStatsChanged)
     Q_PROPERTY(EventStats unreadStats READ unreadStats NOTIFY unreadStatsChanged)
-    Q_PROPERTY(bool allHistoryLoaded READ allHistoryLoaded NOTIFY addedMessages
+    Q_PROPERTY(bool allHistoryLoaded READ allHistoryLoaded NOTIFY allHistoryLoadedChanged
                    STORED false)
     Q_PROPERTY(QStringList tagNames READ tagNames NOTIFY tagsChanged)
     Q_PROPERTY(bool isFavourite READ isFavourite NOTIFY tagsChanged STORED false)
     Q_PROPERTY(bool isLowPriority READ isLowPriority NOTIFY tagsChanged STORED false)
 
-    Q_PROPERTY(GetRoomEventsJob* eventsHistoryJob READ eventsHistoryJob NOTIFY
-                   eventsHistoryJobChanged)
+    Q_PROPERTY(GetRoomEventsJob* eventsHistoryJob READ eventsHistoryJob NOTIFY eventsHistoryJobChanged)
+    Q_PROPERTY(int requestedHistorySize READ requestedHistorySize NOTIFY eventsHistoryJobChanged)
 
     Q_PROPERTY(QStringList accountDataEventTypes READ accountDataEventTypes NOTIFY accountDataChanged)
 
@@ -191,13 +186,9 @@ public:
     enum class Change : quint32 { // QFlags can't go more than 32-bit
         None = 0x0, //!< No changes occurred in the room
         RoomNames = 0x1, //!< \sa namesChanged, displaynameChanged
-        DECL_DEPRECATED_ENUMERATOR(Name, RoomNames),
-        DECL_DEPRECATED_ENUMERATOR(Aliases, RoomNames),
-        DECL_DEPRECATED_ENUMERATOR(CanonicalAlias, RoomNames),
-        // Aliases/CanonicalAlias pre-0.8 = 0x2,
+        // NotInUse = 0x2,
         Topic = 0x4, //!< \sa topicChanged
         PartiallyReadStats = 0x8, //!< \sa partiallyReadStatsChanged
-        DECL_DEPRECATED_ENUMERATOR(UnreadNotifs, PartiallyReadStats),
         Avatar = 0x10, //!< \sa avatarChanged
         JoinState = 0x20, //!< \sa joinStateChanged
         Tags = 0x40, //!< \sa tagsChanged
@@ -205,13 +196,9 @@ public:
         //!     displaynameChanged
         Members = 0x80,
         UnreadStats = 0x100, //!< \sa unreadStatsChanged
-        AccountData Q_DECL_ENUMERATOR_DEPRECATED_X(
-            "Change::AccountData will be merged into Change::Other in 0.8") =
-            0x200,
+        // AccountData pre-0.9 = 0x200,
         Summary = 0x400, //!< \sa summaryChanged, displaynameChanged
-        ReadMarker Q_DECL_ENUMERATOR_DEPRECATED_X(
-            "Change::ReadMarker will be merged into Change::Other in 0.8") =
-            0x800,
+        // ReadMarker pre-0.9 = 0x800,
         Highlights = 0x1000, //!< \sa highlightCountChanged
         //! A catch-all value that covers changes not listed above (such as
         //! encryption turned on or the room having been upgraded), as well as
@@ -235,7 +222,9 @@ public:
     // Property accessors
 
     Connection* connection() const;
-    User* localUser() const;
+
+    //! Get a RoomMember object for the local user.
+    RoomMember localMember() const;
     const QString& id() const;
     QString version() const;
     bool isUnstable() const;
@@ -271,20 +260,15 @@ public:
     QUrl avatarUrl() const;
     const Avatar& avatarObject() const;
     Q_INVOKABLE JoinState joinState() const;
-    Q_INVOKABLE QList<Quotient::User*> usersTyping() const;
-    QList<User*> membersLeft() const;
 
-    Q_INVOKABLE QList<Quotient::User*> users() const;
-    Q_DECL_DEPRECATED_X("Use safeMemberNames() or htmlSafeMemberNames() instead") //
-    QStringList memberNames() const;
-    QStringList safeMemberNames() const;
-    QStringList htmlSafeMemberNames() const;
     int timelineSize() const;
     bool usesEncryption() const;
     RoomEventPtr decryptMessage(const EncryptedEvent& encryptedEvent);
     void handleRoomKeyEvent(const RoomKeyEvent& roomKeyEvent,
                             const QString& senderId,
-                            const QByteArray& olmSessionId);
+                            const QByteArray& olmSessionId,
+                            const QByteArray& senderKey,
+                            const QByteArray& senderEdKey);
     int joinedCount() const;
     int invitedCount() const;
     int totalMemberCount() const;
@@ -306,27 +290,40 @@ public:
      */
     Q_INVOKABLE QImage avatar(int width, int height);
 
-    /**
-     * \brief Get a user object for a given user id
-     * This is the recommended way to get a user object in a room
-     * context. The actual object type may be changed in further
-     * versions to provide room-specific user information (display name,
-     * avatar etc.).
-     * \note The method will return a valid user regardless of
-     *       the membership.
-     */
-    Q_INVOKABLE Quotient::User* user(const QString& userId) const;
+    //! \brief Get a RoomMember object for the given user Matrix ID
+    //!
+    //! Will return a nullptr if there is no m.room.member event for the user in
+    //! the room so needs to be null checked.
+    //!
+    //! \note This can return a member in any state that is known to the room so
+    //!       check the state (using RoomMember::membershipState()) before use.
+    Q_INVOKABLE RoomMember member(const QString& userId) const;
 
-    /**
-     * \brief Check the join state of a given user in this room
-     *
-     * \note Banned and invited users are not tracked separately for now (Leave
-     *       will be returned for them).
-     *
-     * \return Join if the user is a room member; Leave otherwise
-     */
-    Q_DECL_DEPRECATED_X("Use isMember() instead")
-    Q_INVOKABLE Quotient::JoinState memberJoinState(Quotient::User* user) const;
+    //! Get a list of room members who have joined the room.
+    QList<RoomMember> joinedMembers() const;
+
+    //! Get a list of all members known to the room.
+    QList<RoomMember> members() const;
+
+    //! Get a list of all members known to have left the room.
+    QList<RoomMember> membersLeft() const;
+
+    //! Get a list of room members who are currently sending a typing indicator.
+    QList<RoomMember> membersTyping() const;
+
+    //! \brief Get a list of room members who are currently sending a typing indicator.
+    //!
+    //! The local member is excluded from this list.
+    QList<RoomMember> otherMembersTyping() const;
+
+    //! Get a list of room member Matrix IDs who have joined the room.
+    QStringList joinedMemberIds() const;
+
+    //! Get a list of all member Matrix IDs known to the room.
+    QStringList memberIds() const;
+
+    //! Whether the name for the given member should be disambiguated
+    bool needsDisambiguation(const QString& userId) const;
 
     //! \brief Check the join state of a given user in this room
     //!
@@ -336,66 +333,43 @@ public:
     //! Check whether a user with the given id is a member of the room
     Q_INVOKABLE bool isMember(const QString& userId) const;
 
-    //! \brief Get a display name (without disambiguation) for the given member
+    const Avatar& memberAvatarObject(const QString& memberId) const;
+
+    //! \brief Get a avatar of the specified dimensions
     //!
-    //! \sa safeMemberName, htmlSafeMemberName
-    Q_INVOKABLE QString memberName(const QString& mxId) const;
+    //! This always returns immediately; if there's no avatar cached yet, the call triggers
+    //! a network request, that will emit Room::memberAvatarUpdated() once completed.
+    //! \return a pixmap with the avatar or a placeholder if there's none available yet
+    Q_INVOKABLE QImage memberAvatar(const QString& memberId, int width, int height);
 
-    //! \brief Get a disambiguated name for the given user in the room context
-    Q_DECL_DEPRECATED_X("Use safeMemberName() instead")
-    Q_INVOKABLE QString roomMembername(const Quotient::User* u) const;
-    //! \brief Get a disambiguated name for a user with this id in the room
-    Q_DECL_DEPRECATED_X("Use safeMemberName() instead")
-    Q_INVOKABLE QString roomMembername(const QString& userId) const;
-
-    /*!
-     * \brief Get a disambiguated name for the member with the given MXID
-     *
-     * This function should only be used for non-UI code; consider using
-     * safeMemberName() or htmlSafeMemberName() for displayed strings.
-     */
-    Q_INVOKABLE QString disambiguatedMemberName(const QString& mxId) const;
-
-    /*! Get a display-safe member name in the context of this room
-     *
-     * Display-safe means disambiguated and without RLO/LRO markers
-     * (see https://github.com/quotient-im/Quaternion/issues/545).
-     */
-    Q_INVOKABLE QString safeMemberName(const QString& userId) const;
-
-    /*! Get an HTML-safe member name in the context of this room
-     *
-     * This function adds HTML escaping on top of safeMemberName() safeguards.
-     */
-    Q_INVOKABLE QString htmlSafeMemberName(const QString& userId) const;
-
-    //! \brief Get an avatar for the member with the given MXID
-    QUrl memberAvatarUrl(const QString& mxId) const;
+    //! \brief Get a square avatar of the specified size
+    //!
+    //! This is an overload for the case when the needed width and height are equal.
+    Q_INVOKABLE QImage memberAvatar(const QString& memberId, int dimension);
 
     const Timeline& messageEvents() const;
     const PendingEvents& pendingEvents() const;
 
-    /// Check whether all historical messages are already loaded
-    /**
-     * \return true if the "oldest" event in the timeline is
-     *         a room creation event and there's no further history
-     *         to load; false otherwise
-     */
+    //! \brief Get the number of requested historical events
+    //! \return The number of requested events if there's a pending request; 0 otherwise
+    int requestedHistorySize() const;
+
+    //! Check whether all historical messages are already loaded
+    //! \return true if the "oldest" event in the timeline is a room creation event and there's
+    //!         no further history to load; false otherwise
     bool allHistoryLoaded() const;
-    /**
-     * A convenience method returning the read marker to the position
-     * before the "oldest" event; same as messageEvents().crend()
-     */
+
+    //! \brief Get a reverse iterator at the position before the "oldest" event
+    //!
+    //! Same as messageEvents().crend()
     rev_iter_t historyEdge() const;
-    /**
-     * A convenience method returning the iterator beyond the latest
-     * arrived event; same as messageEvents().cend()
-     */
+    //! \brief Get an iterator for the position beyond the latest arrived event
+    //!
+    //! Same as messageEvents().cend()
     Timeline::const_iterator syncEdge() const;
     Q_INVOKABLE Quotient::TimelineItem::index_t minTimelineIndex() const;
     Q_INVOKABLE Quotient::TimelineItem::index_t maxTimelineIndex() const;
-    Q_INVOKABLE bool
-    isValidIndex(Quotient::TimelineItem::index_t timelineIndex) const;
+    Q_INVOKABLE bool isValidIndex(Quotient::TimelineItem::index_t timelineIndex) const;
 
     rev_iter_t findInTimeline(TimelineItem::index_t index) const;
     rev_iter_t findInTimeline(const QString& evtId) const;
@@ -428,37 +402,6 @@ public:
     void setLastDisplayedEventId(const QString& eventId);
     void setLastDisplayedEvent(TimelineItem::index_t index);
 
-    //! \brief Obtain a read receipt of any user
-    //! \deprecated Use lastReadReceipt or fullyReadMarker instead.
-    //!
-    //! Historically, readMarker was returning a "converged" read marker
-    //! representing both the read receipt and the fully read marker, as
-    //! Quotient managed them together. Since 0.6.8, a single-argument call of
-    //! readMarker returns the last read receipt position (for any room member)
-    //! and a call without arguments returns the last _fully read_ position,
-    //! to provide access to both positions separately while maintaining API
-    //! stability guarantees. 0.7 has separate methods to return read receipts
-    //! and the fully read marker - use them instead.
-    //! \sa lastReadReceipt
-    [[deprecated("Use lastReadReceipt() to get m.read receipt or"
-                 " fullyReadMarker() to get m.fully_read marker")]] //
-    rev_iter_t readMarker(const User* user) const;
-    //! \brief Obtain the local user's fully-read marker
-    //! \deprecated Use fullyReadMarker instead
-    //!
-    //! See the documentation for the single-argument overload.
-    //! \sa fullyReadMarker
-    [[deprecated("Use localReadReceiptMarker() or fullyReadMarker()")]] //
-    rev_iter_t readMarker() const;
-    //! \brief Get the event id for the local user's fully-read marker
-    //! \deprecated Use lastFullyReadEventId instead
-    //!
-    //! See the readMarker documentation
-    [[deprecated("Use lastReadReceipt() to get m.read receipt or"
-                 " lastFullyReadEventId() to get an event id that"
-                 " m.fully_read marker points to")]] //
-    QString readMarkerEventId() const;
-
     //! \brief Get the latest read receipt from a user
     //!
     //! The user id must be valid. A read receipt with an empty event id
@@ -514,11 +457,6 @@ public:
     //! \sa lastReadReceipt, allMembersLoaded
     QSet<QString> userIdsAtEvent(const QString& eventId) const;
 
-    QSet<QString> userIdsAtEvent(const QString& eventId); // See #706
-
-    [[deprecated("Use userIdsAtEvent instead")]]
-    QSet<User*> usersAtEventId(const QString& eventId);
-
     //! \brief Mark the event with uptoEventId as fully read
     //!
     //! Marks the event with the specified id as fully read locally and also
@@ -598,36 +536,6 @@ public:
     //!     highlightCount
     EventStats unreadStats() const;
 
-    [[deprecated(
-        "Use partiallyReadStats/unreadStats() and EventStats::empty()")]]
-    bool hasUnreadMessages() const;
-
-    //! \brief Get the number of notable events since the fully read marker
-    //!
-    //! \deprecated Since 0.7 there are two ways to count unread events: since
-    //! the fully read marker (used by libQuotient pre-0.7) and since the last
-    //! read receipt (as used by most of Matrix ecosystem, including the spec
-    //! and MSCs). This function currently returns a value derived from
-    //! partiallyReadStats() for compatibility with libQuotient 0.6; it will be
-    //! removed due to ambiguity. Use unreadStats() to obtain the spec-compliant
-    //! count of unread events and the highlight count; partiallyReadStats() to
-    //! obtain the unread events count since the fully read marker.
-    //!
-    //! \return -1 (_not 0_) when all messages are known to have been fully read,
-    //!         i.e. the fully read marker points to _the latest notable_ event
-    //!         loaded in the local timeline (which may be different from
-    //!         the latest event in the local timeline as that might not be
-    //!         notable);
-    //!         0 when there may be unread messages but the current local
-    //!         timeline doesn't have any notable ones (often but not always
-    //!         because it's entirely empty yet);
-    //!         a positive integer when there is (or estimated to be) a number
-    //!         of unread notable events as described above.
-    //!
-    //! \sa partiallyReadStats, unreadStats
-    [[deprecated("Use partiallyReadStats() or unreadStats() instead")]] //
-    int unreadCount() const;
-
     //! \brief Get the number of notifications since the last read receipt
     //!
     //! This is the same as <tt>unreadStats().notableCount</tt>.
@@ -661,7 +569,7 @@ public:
 
     QStringList tagNames() const;
     TagsMap tags() const;
-    TagRecord tag(const QString& name) const;
+    Tag tag(const QString& name) const;
 
     /** Add a new tag to this room
      * If this room already has this tag, nothing happens. If it's a new
@@ -669,7 +577,7 @@ public:
      * of tags and the new set is sent to the server to update other
      * clients.
      */
-    void addTag(const QString& name, const TagRecord& record = {});
+    void addTag(const QString& name, const Tag& tagData = {});
     Q_INVOKABLE void addTag(const QString& name, float order);
 
     /// Remove a tag from the room
@@ -711,8 +619,8 @@ public:
     /// Check whether this room is a direct chat
     Q_INVOKABLE bool isDirectChat() const;
 
-    /// Get the list of users this room is a direct chat with
-    QList<User*> directChatUsers() const;
+    /// Get the list of members this room is a direct chat with
+    QList<RoomMember> directChatMembers() const;
 
     Q_INVOKABLE QUrl makeMediaUrl(const QString& eventId,
                                   const QUrl &mxcUrl) const;
@@ -751,64 +659,92 @@ public:
      */
     Q_INVOKABLE QString prettyPrint(const QString& plainText) const;
 
+#if Quotient_VERSION_MAJOR == 0 && Quotient_VERSION_MINOR < 10
+    [[deprecated("Create MemberSorter objects directly instead")]]
     MemberSorter memberSorter() const;
+#endif
 
     Q_INVOKABLE bool supportsCalls() const;
 
     /// Whether the current user is allowed to upgrade the room
     Q_INVOKABLE bool canSwitchVersions() const;
 
-    /// Get a state event with the given event type and state key
-    /*! This method returns a (potentially empty) state event corresponding
-     * to the pair of event type \p evtType and state key \p stateKey.
-     */
-    [[deprecated("Use currentState().get() instead; "
-                 "make sure to check its result for nullptrs")]] //
-    const StateEvent* getCurrentState(const QString& evtType,
-                                      const QString& stateKey = {}) const;
-
-    /// Get a state event with the given event type and state key
-    /*! This is a typesafe overload that accepts a C++ event type instead of
-     * its Matrix name.
-     */
-    template <typename EvT>
-    [[deprecated("Use currentState().get() instead; "
-                 "make sure to check its result for nullptrs")]] //
-    const EvT* getCurrentState(const QString& stateKey = {}) const
-    {
-        QT_IGNORE_DEPRECATIONS(const auto* evt = eventCast<const EvT>(
-                                   getCurrentState(EvT::TypeId, stateKey));)
-        Q_ASSERT(evt);
-        Q_ASSERT(evt->matrixType() == EvT::TypeId
-                 && evt->stateKey() == stateKey);
-        return evt;
-    }
-
     /// \brief Get the current room state
     RoomStateView currentState() const;
 
+    //! \brief The effective power level of the given member in the room
+    //!
+    //! This is normally the same as calling `RoomPowerLevelEvent::powerLevelForUser(userId)` but
+    //! takes into account the room context and works even if the room state has no power levels
+    //! event. It is THE recommended way to get a room member's power level to display in the UI.
+    //! \param memberId The room member ID to check; if empty, the local user will be checked
+    //! \sa RoomPowerLevelsEvent, https://spec.matrix.org/v1.11/client-server-api/#mroompower_levels
+    Q_INVOKABLE int memberEffectivePowerLevel(const QString& memberId = {}) const;
+
+    //! \brief Get the power level required to send events of the given type
+    //!
+    //! \note This is a generic method that only gets the power level to send events with a given
+    //!       type. Some operations have additional restrictions or enablers though: e.g.,
+    //!       room member changes (kicks, invites) have special power levels; on the other hand,
+    //!       redactions of one's own messages are allowed regardless of the power level. To check
+    //!       effective ability to perform an operation, use Room's can*() methods instead of
+    //!       comparing the power levels (those are also slightly more efficient).
+    //! \note Unlike the template version below, this method determines at runtime whether an event
+    //!       type is that of a state event, assuming unknown event types to be non-state; pass
+    //!       `true` as the second parameter to override that.
+    //! \sa canSend, canRedact, canSwitchVersions
+    Q_INVOKABLE int powerLevelFor(const QString& eventTypeId, bool forceStateEvent = false) const;
+
+    //! \brief Get the power level required to send events of the given type
+    //!
+    //! This is an optimised version of non-template powerLevelFor() (with the same caveat about
+    //! operations based on some event types) for cases when the event type is known at build time.
+    //! \tparam EvT the event type to get the power level for
+    template <EventClass EvT>
+    int powerLevelFor() const
+    {
+        return currentState().get<RoomPowerLevelsEvent>()->powerLevelForEventType<EvT>();
+    }
+
+    //! \brief Post a pre-created room message event
+    //!
+    //! Takes ownership of the event, deleting it once the matching one arrives with the sync.
+    //! \note Do not assume that the event is already on the road to the homeserver when this (or
+    //!       any other `post*`) method returns; it can be queued internally.
+    //! \sa PendingEventItem::deliveryStatus()
+    //! \return a reference to the pending event item
+    const PendingEventItem& post(RoomEventPtr event);
+
+    template <typename EvT, typename... ArgTs>
+    const PendingEventItem& post(ArgTs&&... args)
+    {
+        return post(makeEvent<EvT>(std::forward<ArgTs>(args)...));
+    }
+
+    QString postFile(const QString& plainText,
+                     std::unique_ptr<EventContent::FileContentBase> fileContent);
+
+    PendingEventItem::future_type whenMessageMerged(QString txnId) const;
+
     //! Send a request to update the room state with the given event
     SetRoomStateWithKeyJob* setState(const StateEvent& evt);
 
     //! \brief Set a state event of the given type with the given arguments
     //!
-    //! This typesafe overload attempts to send a state event with the type
-    //! \p EvT and the content defined by \p args. Specifically, the function
-    //! constructs a temporary object of type \p EvT with its content
-    //! list-initialised from \p args, and sends a request to the homeserver
-    //! using the Matrix event type defined by \p EvT and the event content
-    //! produced via EvT::contentJson().
-    //!
-    //! \note This call is not suitable for events that assume non-empty
-    //!       stateKey, such as member events; for those you have to create
-    //!       a temporary event object yourself and use the setState() overload
-    //!       that accepts StateEvent const-ref.
+    //! This type-safe overload attempts to send a state event of the type \p EvT constructed from
+    //! \p args.
     template <typename EvT, typename... ArgTs>
     auto setState(ArgTs&&... args)
     {
         return setState(EvT(std::forward<ArgTs>(args)...));
     }
 
+    void addMegolmSessionFromBackup(const QByteArray &sessionId, const QByteArray &sessionKey, uint32_t index, const QByteArray& senderKey, const QByteArray& senderEdKey);
+
+    Q_INVOKABLE void startVerification();
+
+    QJsonArray exportMegolmSessions();
+
 public Q_SLOTS:
     /** Check whether the room should be upgraded */
     void checkVersion();
@@ -821,18 +757,13 @@ public Q_SLOTS:
     /// Send a reaction on a given event with a given key
     QString postReaction(const QString& eventId, const QString& key);
 
-    QString postFile(const QString& plainText, EventContent::TypedBase* content);
-#if QT_VERSION_MAJOR < 6
-    Q_DECL_DEPRECATED_X("Use postFile(QString, MessageEventType, EventContent)") //
-    QString postFile(const QString& plainText, const QUrl& localPath,
-                     bool asGenericFile = false);
-#endif
     /** Post a pre-created room message event
      *
      * Takes ownership of the event, deleting it once the matching one
      * arrives with the sync
      * \return transaction id associated with the event.
      */
+    [[deprecated("Use post() instead")]]
     QString postEvent(RoomEvent* event);
     QString postJson(const QString& matrixType, const QJsonObject& eventContent);
     QString retryMessage(const QString& txnId);
@@ -852,10 +783,10 @@ public Q_SLOTS:
     /// You shouldn't normally call this method; it's here for debugging
     void refreshDisplayName();
 
-    void getPreviousContent(int limit = 10, const QString &filter = {});
+    JobHandle<GetRoomEventsJob> getPreviousContent(int limit = 10, const QString &filter = {});
 
     void inviteToRoom(const QString& memberId);
-    LeaveRoomJob* leaveRoom();
+    JobHandle<LeaveRoomJob> leaveRoom();
     void kickMember(const QString& memberId, const QString& reason = {});
     void ban(const QString& userId, const QString& reason = {});
     void unban(const QString& userId);
@@ -881,9 +812,6 @@ public Q_SLOTS:
     void inviteCall(const QString& callId, const int lifetime,
                     const QString& sdp);
     void sendCallCandidates(const QString& callId, const QJsonArray& candidates);
-    [[deprecated("Lifetime argument is no more passed; "
-                 "use 2-arg Room::answerCall() instead")]]
-    void answerCall(const QString& callId, int lifetime, const QString& sdp);
     void answerCall(const QString& callId, const QString& sdp);
     void hangupCall(const QString& callId);
 
@@ -910,7 +838,7 @@ Q_SIGNALS:
     /// The event is about to be appended to the list of pending events
     void pendingEventAboutToAdd(Quotient::RoomEvent* event);
     /// An event has been appended to the list of pending events
-    void pendingEventAdded();
+    void pendingEventAdded(const Quotient::RoomEvent* event);
     /// The remote echo has arrived with the sync and will be merged
     /// with its local counterpart
     /** NB: Requires a sync loop to be emitted */
@@ -955,11 +883,28 @@ Q_SIGNALS:
     void pinnedEventsChanged();
     void topicChanged();
     void avatarChanged();
-    void userAdded(Quotient::User* user);
-    void userRemoved(Quotient::User* user);
-    void memberAboutToRename(Quotient::User* user, QString newName);
-    void memberRenamed(Quotient::User* user);
-    void memberAvatarChanged(Quotient::User* user);
+
+    //! \brief A new member has joined the room
+    //!
+    //! This can be from any previous state or a member previously unknown to
+    //! the room.
+    void memberJoined(RoomMember member);
+
+    //! \brief A member who previously joined has left
+    //!
+    //! The member will still be known to the room their membership state has changed
+    //! from Membership::Join to anything else.
+    void memberLeft(RoomMember member);
+
+    //! A known joined member is about to update their display name
+    void memberNameAboutToUpdate(RoomMember member, QString newName);
+
+    //! A known joined member has updated their display name
+    void memberNameUpdated(RoomMember member);
+
+    //! A known joined member has updated their avatar
+    void memberAvatarUpdated(RoomMember member);
+
     /// The list of members has changed
     /** Emitted no more than once per sync, this is a good signal to
      * for cases when some action should be done upon any change in
@@ -968,6 +913,7 @@ Q_SIGNALS:
      * instead.
      */
     void memberListChanged();
+
     /// The previously lazy-loaded members list is now loaded entirely
     /// \sa setDisplayed
     void allMembersLoaded();
@@ -975,6 +921,8 @@ Q_SIGNALS:
 
     void joinStateChanged(Quotient::JoinState oldState,
                           Quotient::JoinState newState);
+
+    //! The list of members sending typing indicators has changed.
     void typingChanged();
 
     void highlightCountChanged(); ///< \sa highlightCount
@@ -987,16 +935,9 @@ Q_SIGNALS:
     //! \sa lastReadReceipt
     void lastReadEventChanged(QVector<QString> userIds);
     void fullyReadMarkerMoved(QString fromEventId, QString toEventId);
-    [[deprecated("Since 0.7, use fullyReadMarkerMoved")]]
-    void readMarkerMoved(QString fromEventId, QString toEventId);
-    [[deprecated("Since 0.7, use lastReadEventChanged")]]
-    void readMarkerForUserMoved(Quotient::User* user, QString fromEventId,
-                                QString toEventId);
-    [[deprecated("Since 0.7, use either partiallyReadStatsChanged "
-                 "or unreadStatsChanged")]]
-    void unreadMessagesChanged(Quotient::Room* room);
     void partiallyReadStatsChanged();
     void unreadStatsChanged();
+    void allHistoryLoadedChanged();
 
     void accountDataAboutToChange(QString type);
     void accountDataChanged(QString type);
@@ -1053,22 +994,22 @@ private:
     void setJoinState(JoinState state);
 };
 
-class QUOTIENT_API MemberSorter {
-public:
-    explicit MemberSorter(const Room* r) : room(r) {}
+template <template <class> class ContT>
+inline typename ContT<RoomMember>::size_type lowerBoundMemberIndex(const ContT<RoomMember>& c,
+                                                                   const auto& v,
+                                                                   MemberSorter ms = {})
+{
+    return std::ranges::lower_bound(c, v, ms) - c.begin();
+}
 
-    bool operator()(User* u1, User* u2) const;
-    bool operator()(User* u1, QStringView u2name) const;
+template <template <class> class ContT>
+inline typename ContT<QString>::size_type lowerBoundMemberIndex(const ContT<QString>& c,
+                                                                const auto& v, const Room* r,
+                                                                MemberSorter ms = {})
+{
+    return std::ranges::lower_bound(c, v, ms, std::bind_front(&Room::member, r)) - c.begin();
+}
 
-    template <typename ContT, typename ValT>
-    typename ContT::size_type lowerBoundIndex(const ContT& c, const ValT& v) const
-    {
-        return std::lower_bound(c.begin(), c.end(), v, *this) - c.begin();
-    }
-
-private:
-    const Room* room;
-};
 } // namespace Quotient
 Q_DECLARE_METATYPE(Quotient::FileTransferInfo)
 Q_DECLARE_METATYPE(Quotient::ReadReceipt)
